/*

Copyright (C) 2019-23 Dirk Schuetz <dirk.schuetz@durham.ac.uk>

This file is part of KnotJob.

KnotJob is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

KnotJob is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org.licenses/>.

 */

package knotjob;

import knotjob.frames.GridDiagramFrame;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Desktop;    // to comment out for Java problems
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Taskbar;    // to comment out for Java problems
import java.awt.Toolkit;
import java.awt.desktop.AboutEvent;   // to comment out for Java problems
import java.awt.desktop.AboutHandler; // to comment out for Java problems
import java.awt.desktop.QuitEvent;    // to comment out for Java problems
import java.awt.desktop.QuitHandler;  // to comment out for Java problems
import java.awt.desktop.QuitResponse; // to comment out for Java problems
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collections;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import knotjob.diagrams.ShowDiagram;
import knotjob.diagrams.griddiagrams.ShowGridDiagram;
import knotjob.links.*;
import knotjob.dialogs.*;
import knotjob.filters.Filter;
import knotjob.frames.StringFrame;
import knotjob.frames.ViewBLTInvariant;
import knotjob.homology.HomologyInfo;
import knotjob.frames.ViewCohomology;
import knotjob.frames.ViewDocumentation;
import knotjob.frames.ViewOddHomology;
import knotjob.frames.ViewSInvariants;
import knotjob.frames.ViewStableType;
import knotjob.homology.QuantumCohomology;
import knotjob.polynomial.HalfPolynomial;

/**
 *
 * @author Dirk
 */
public class Knobster extends JFrame {

    private final DefaultListModel<String> listModelAll;
    private final DefaultListModel<String> listModelFiltered;
    protected final ArrayList<LinkData> allLinks;
    protected final ArrayList<LinkData> filteredLinks;
    private final JList<String> list;
    protected int[] choices;
    private int choice;
    private final JPanel panelLinks;
    private final JPanel panelKnotInfo;
    private final JPanel panelButtons;
    private final KnotButtons theButtons;
    private JLabel labelLinkNumber;
    protected boolean filtered;
    public final Options options;
    private final Comparer comparer;
    private final Image img;
    private final ArrayList<Filter> existingFilters;
    private Filter activeFilter;
    private FilterBusiness filterBusiness;
    private final Dimension panelDimSm = new Dimension(700, 24);
    private final Dimension panelDimLg = new Dimension(700, 32);

    public Knobster(String title, Options optns) {
        super(title);
        options = optns;
        img = options.getImage();
        System.setProperty("apple.laf.useScreenMenuBar", "true");
        listModelAll = new DefaultListModel<String>();
        listModelFiltered = new DefaultListModel<String>();
        allLinks = new ArrayList<LinkData>();
        filteredLinks = new ArrayList<LinkData>();
        list = new JList<String>(listModelAll);
        choices = null;
        choice = -1;
        panelLinks = new JPanel();
        panelKnotInfo = new JPanel();
        panelButtons = new JPanel();
        theButtons = new KnotButtons(options, this);
        filtered = false;
        existingFilters = new ArrayList<Filter>();
        comparer = new Comparer(0);
        arrangeStuff();
    }

    public void setAbout() {
        /*if (Desktop.isDesktopSupported()) {
            Desktop desktop = Desktop.getDesktop(); // // to comment out for Java 8 with /*
            desktop.setAboutHandler(new AboutHandler() {
                @Override
                public void handleAbout(AboutEvent e) {
                    showInfo();
                }
            });
            desktop.setQuitHandler(new QuitHandler() {
                @Override
                public void handleQuitRequestWith(QuitEvent e, QuitResponse response) {
                    //if (yesNoDialog("Quit KnotJob", Color.CYAN))
                        System.exit(0);
                }
            });
        }// */ //to comment out for Java 8
    }

    public void setClosing() {
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent arg0) {
                if (yesNoDialog("Quit KnotJob", Color.ORANGE))
                    System.exit(0);
            }
        });
    }

    public void setIcon() {
        if (img != null) setIconImage(img);
        /*if (Taskbar.isTaskbarSupported()) {             // to comment out for Java 8
            Taskbar taskbar = Taskbar.getTaskbar();     // to comment out for Java 8
            if (img != null) taskbar.setIconImage(img); // to comment out for Java 8
        }// */
    }

    private void arrangeStuff() {
        addMenus();
        addLinkList();
        addFilters();
        setKnotInfoEmpty();
    }

    private void addFilters() {
        filterBusiness = new FilterBusiness(existingFilters, options,
                this, listModelAll, allLinks);
        activeFilter = filterBusiness.resetFilters();
    }

    private void addMenus() {
        JMenuBar menubar = new JMenuBar();
        addFiletoMenu(menubar);
        addEdittoMenu(menubar);
        addCalctoMenu(menubar);
        addFilttoMenu(menubar);
        addDiagramtoMenu(menubar);
        addHelptoMenu(menubar);
        this.setJMenuBar(menubar);
    }

    private void addFiletoMenu(JMenuBar menubar) {
        JMenu file = new JMenu("File");
        JMenuItem newlink = new JMenuItem("New Link");
        JMenuItem lodlink = new JMenuItem("Open Link(s)");
        JMenuItem lsdlink = new JMenuItem("Open selected Link(s)");
        JMenuItem savlink = new JMenuItem("Save Link(s)");
        JMenuItem explink = new JMenuItem("Export Link(s)");
        JMenuItem quilink = new JMenuItem("Quit");
        if (options.getOperatingSystem() != 2) {
            quilink.setMnemonic(KeyEvent.VK_Q);
            quilink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
            // replace the previous line with the line below for Java 8
            //quilink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        }
        quilink.addActionListener(new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent ae) {
                //if (yesNoDialog("Quit KnotJob", Color.GREEN))
                    System.exit(0);
            }
        });
        lodlink.setMnemonic(KeyEvent.VK_O);
        lodlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        //lodlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        lodlink.addActionListener(new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent ae) {
                ArrayList<LinkData> links = loadLinks(false);
                for (LinkData ink : links) {
                    allLinks.add(ink);
                    listModelAll.addElement(ink.name);
                }
                if (!filtered) labelLinkNumber.setText("Links : "+allLinks.size());
                else labelLinkNumber.setText("Links : "+filteredLinks.size());
            }
        });// */
        //lsdlink.setMnemonic(KeyEvent.VK_E);
        //lsdlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        //lsdlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_SHIFT,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        lsdlink.addActionListener(new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent ae) {
                ArrayList<LinkData> links = loadLinks(true);
                for (LinkData ink : links) {
                    allLinks.add(ink);
                    listModelAll.addElement(ink.name);
                }
                if (!filtered) labelLinkNumber.setText("Links : "+allLinks.size());
                else labelLinkNumber.setText("Links : "+filteredLinks.size());
            }
        });
        newlink.setMnemonic(KeyEvent.VK_N);
        newlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // newlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        newlink.addActionListener(new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent ae) {
                ArrayList<LinkData> inks = getNewLink();
                for (LinkData ink : inks) {
                    if (ink != null) {
                        allLinks.add(ink);
                        listModelAll.addElement(ink.name);
                    }
                }
                if (!filtered) labelLinkNumber.setText("Links : "+allLinks.size());// */
                else labelLinkNumber.setText("Links : "+filteredLinks.size());
            }
        });// */
        savlink.setMnemonic(KeyEvent.VK_S);
        savlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // savlink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        savlink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                if (choices != null) saveLinks();
            }
        });// */
        explink.setMnemonic(KeyEvent.VK_E);
        explink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // explink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        explink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                if (choices != null) exportLinks();
            }
        });
        file.add(newlink);
        file.add(lodlink);
        file.add(lsdlink);
        file.add(savlink);
        file.addSeparator();
        file.add(explink);
        file.addSeparator();
        file.add(quilink);
        menubar.add(file);
    }

    private void addEdittoMenu(JMenuBar menubar) {
        JMenu edit = new JMenu("Edit");
        JMenuItem editlink = new JMenuItem("Edit Link");
        JMenuItem deletelink = new JMenuItem("Remove Link(s)");
        JMenuItem chosallink = new JMenuItem("Select All");
        JMenuItem optionlink = new JMenuItem("Options");
        editlink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choice >= 0) editLink();
            }
        });
        deletelink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choice >= 0 && yesNoDialog("Remove selected links", Color.BLUE)) {
                    setCursor(new Cursor(Cursor.WAIT_CURSOR));
                    ArrayList<LinkData> linkList = allLinks;
                    if (filtered) linkList = filteredLinks;
                    int i = choices.length-1;
                    while (i >= 0) {
                        LinkData toRemove = linkList.get(choices[i]);
                        listModelAll.removeElementAt(allLinks.indexOf(toRemove));
                        allLinks.remove(toRemove);
                        if (filteredLinks.contains(toRemove)) {
                            listModelFiltered.removeElementAt(filteredLinks.indexOf(toRemove));
                            filteredLinks.remove(toRemove);
                        }
                        i--;
                    }
                    if (filtered) list.setModel(listModelFiltered);
                    else list.setModel(listModelAll);
                    setKnotInfoEmpty();
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
            }
        });
        chosallink.setMnemonic(KeyEvent.VK_A);
        chosallink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // chosallink.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        chosallink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                setCursor(new Cursor(Cursor.WAIT_CURSOR));
                int n = allLinks.size();
                if (filtered) n = filteredLinks.size();
                int[] all = new int[n];
                for (int i = 0; i < all.length; i++) all[i]=i;
                list.setSelectedIndices(all);
                choices = all;
                if (choices.length > 0) choice = 0;
                setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        });
        optionlink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                setOptions(0);
            }
        });
        edit.add(editlink);
        edit.add(deletelink);
        edit.add(chosallink);
        edit.addSeparator();
        edit.add(optionlink);
        menubar.add(edit);
    }

    private void addCalctoMenu(JMenuBar menubar) {
        JMenu calculations = new JMenu("Calculations");
        JMenuItem sinvs = new JMenuItem("s-Invariants");
        JMenuItem slipsar = new JMenuItem("Lipshitz-Sarkar Invariants");
        JMenuItem sgraded = new JMenuItem("Graded s-Invariants");
        JMenuItem specseq = new JMenuItem("BLT-Spectral Sequence");
        JMenuItem sign = new JMenuItem("Signature");
        JMenuItem khovhom = new JMenuItem("Char 0 Khovanov Cohomology");
        JMenuItem mkhovhom = new JMenuItem("Modular Khovanov Cohomology");
        JMenuItem oddhom = new JMenuItem("Char 0 Odd Khovanov Homology");
        JMenuItem moddhom = new JMenuItem("Modular Odd Khovanov Homology");
        JMenuItem jones = new JMenuItem("Jones Polynomial");
        JMenuItem alex = new JMenuItem("Alexander Polynomial");
        JMenuItem stable = new JMenuItem("Stable Homotopy Type");
        JMenuItem special = new JMenuItem("Special");
        sinvs.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.sInvAction();
            }
        });
        slipsar.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.lipSarAction();
            }
        });
        sgraded.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.gradedAction();
            }
        });
        specseq.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.spectralAction();
            }
        });
        sign.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.signAction();
            }
        });
        khovhom.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.khovhomAction(false);
            }
        });
        mkhovhom.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.khovhomAction(true);
            }
        });
        oddhom.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.oddhomAction(false);
            }
        });
        moddhom.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.oddhomAction(true);
            }
        });
        jones.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.jonesAction();
            }
        });
        alex.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.alexAction();
            }
        });
        stable.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                theButtons.stableAction();
            }
        });
        special.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                specialAction();
            }
        });
        calculations.add(sinvs);
        calculations.add(slipsar);
        calculations.add(sgraded);
        calculations.add(specseq);
        calculations.add(sign);
        calculations.addSeparator();
        calculations.add(khovhom);
        calculations.add(mkhovhom);
        calculations.add(oddhom);
        calculations.add(moddhom);
        calculations.addSeparator();
        calculations.add(jones);
        calculations.add(alex);
        calculations.addSeparator();
        calculations.add(stable);
        //calculations.add(special);
        menubar.add(calculations);
    }

    private void addFilttoMenu(JMenuBar menubar) {
        JMenu sorter = new JMenu("Filters");
        JMenuItem sortlinks = new JMenuItem("Sort Links");
        JMenuItem creafilter = new JMenuItem("Create Filter");
        JMenuItem selfilter = new JMenuItem("Select Filter");
        JMenuItem recfilter = new JMenuItem("Recalculate Filter");
        JMenuItem editfilter = new JMenuItem("Edit Filter Name");
        JMenuItem remfilter = new JMenuItem("Remove Filter");
        selfilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                filterBusiness.selectFilters();
            }
        });
        creafilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                filterBusiness.createFilter();
            }
        });
        recfilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                setFilter();
            }
        });
        editfilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                filterBusiness.editFilter();
            }
        });
        remfilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                filterBusiness.removeFilter();
            }
        });
        sortlinks.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                sortLinks();
            }
        });
        recfilter.setMnemonic(KeyEvent.VK_R);
        recfilter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // recfilter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        selfilter.setMnemonic(KeyEvent.VK_F);
        selfilter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // selfilter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        sorter.add(selfilter);
        sorter.add(creafilter);
        sorter.add(recfilter);
        sorter.add(editfilter);
        sorter.add(remfilter);
        sorter.addSeparator();
        sorter.add(sortlinks);
        menubar.add(sorter);
    }

    private void addDiagramtoMenu(JMenuBar menubar) {
        JMenu diagram = new JMenu("Diagram");
        JMenuItem girthdiag = new JMenuItem("Minimize Girth");
        JMenuItem setdiag = new JMenuItem("Choose Girth-minimized Diagram");
        JMenuItem combdiag = new JMenuItem("Combine Crossings");
        JMenuItem micodiag = new JMenuItem("Minimize combined Crossings");
        JMenuItem selediag = new JMenuItem("Choose minimal combined Diagram");
        JMenuItem optDiag = new JMenuItem("Girth Options");
        setdiag.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choices != null) {
                    ArrayList<LinkData> theLinks = allLinks;
                    if (filtered) theLinks = filteredLinks;
                    for (int i : choices) {
                        LinkData lData = theLinks.get(i);
                        lData.choseGirthMinimized();
                    }
                    if (choices.length > 0) setLinkInfo(theLinks.get(choice));
                }
            }
        });
        girthdiag.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choices != null) {
                    ArrayList<LinkData> linkList = allLinks;
                    if (filtered) linkList = filteredLinks;
                    linkList = chosenLinks(linkList, choices);
                    minimizeGirth(linkList);
                    if (choices.length > 0) setLinkInfo(linkList.get(0));
                }
            }
        });
        girthdiag.setMnemonic(KeyEvent.VK_G);
        girthdiag.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMaskEx()));
        // replace the previous line with the line below for Java 8
        // girthdiag.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G,  Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        combdiag.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choices != null) {
                    ArrayList<LinkData> linkList = allLinks;
                    if (filtered) linkList = filteredLinks;
                    linkList = chosenLinks(linkList, choices);
                    combineCrossings(linkList);
                    if (choices.length > 0) setLinkInfo(linkList.get(0));
                }
            }
        });
        micodiag.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choices != null) {
                    ArrayList<LinkData> linkList = allLinks;
                    if (filtered) linkList = filteredLinks;
                    linkList = chosenLinks(linkList, choices);
                    minimizeDiagramCrossings(linkList);
                    if (choices.length > 0) setLinkInfo(linkList.get(0));
                }
            }
        });

        selediag.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (choices != null) {
                    ArrayList<LinkData> theLinks = allLinks;
                    if (filtered) theLinks = filteredLinks;
                    for (int i : choices) {
                        LinkData lData = theLinks.get(i);
                        lData.choseCombinedMinimized();
                    }
                    if (choices.length > 0) setLinkInfo(theLinks.get(choice));
                }
            }
        });
        optDiag.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                setOptions(1);
            }
        });
        diagram.add(girthdiag);
        diagram.add(setdiag);
        diagram.addSeparator();
        diagram.add(combdiag);
        diagram.add(micodiag);
        diagram.add(selediag);
        diagram.addSeparator();
        diagram.add(optDiag);
        menubar.add(diagram);
    }

    private void addHelptoMenu(JMenuBar menubar) {
        JMenu help = new JMenu("Help");
        JMenuItem helpItem = new JMenuItem("KnotJob Documentation");
        JMenuItem infoItem = new JMenuItem("About KnotJob");
        helpItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                helpInfo();
            }
        });
        infoItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showInfo();
            }
        });
        help.add(helpItem);
        help.addSeparator();
        help.add(infoItem);
        menubar.add(help);
    }

    private void helpInfo() {
        ViewDocumentation viewer = new ViewDocumentation(this, "KnotJob Documentation", options);
        viewer.setUpStuff();
    }

    protected void showInfo() {
        InfoDialog dial = new InfoDialog(null, "About", true, options);
        dial.showInfo();
    }

    protected boolean yesNoDialog(String title, Color col) {
        YesNoDialog dial = new YesNoDialog(this, title, true, options, col);
        return dial.showDialog();
    }

    private void addLinkList() {
        Container box = Box.createVerticalBox();
        JScrollPane scroller = new JScrollPane(list);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        box.setPreferredSize(new Dimension(200,470));
        box.add(scroller);
        panelLinks.setLayout(new BorderLayout());
        panelLinks.add(box, BorderLayout.CENTER);
        labelLinkNumber = new JLabel("Links : 0");
        JCheckBox filteredOk = new JCheckBox("Filter");
        JPanel filterPanel = new JPanel();
        filterPanel.add(labelLinkNumber);
        filterPanel.add(filteredOk);
        filteredOk.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                JCheckBox theBox = (JCheckBox) ae.getSource();
                if (theBox.isSelected()) {
                    filtered = true;
                    list.setModel(listModelFiltered);
                    labelLinkNumber.setText("Links : "+filteredLinks.size());
                }
                else {
                    filtered = false;
                    list.setModel(listModelAll);
                    labelLinkNumber.setText("Links : "+allLinks.size());
                }
                setKnotInfoEmpty();
            }
        });
        list.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent le) {
                @SuppressWarnings("unchecked")
                JList<String> list = (JList<String>) le.getSource();
                choices = list.getSelectedIndices();
                int u = list.getSelectedIndex();
                if ( u >= 0 & u != choice) {
                    ArrayList<LinkData> linkList = allLinks;
                    if (filtered) linkList = filteredLinks;
                    setLinkInfo(linkList.get(u));
                    choice = u;
                }
            }
        });
        panelLinks.add(filterPanel, BorderLayout.SOUTH);
        setUpPanelButtons();
        setLayout(new BorderLayout());
        add(panelLinks, BorderLayout.WEST);
        add(panelKnotInfo, BorderLayout.CENTER);
        add(panelButtons, BorderLayout.SOUTH);
    }

    private void setUpPanelButtons() {
        panelButtons.setPreferredSize(new Dimension(200, 200));
        panelButtons.setBackground(options.getColor());
        theButtons.addButtonsTo(panelButtons);
    }

    private void specialAction() {
        if (choices != null) {
            if (choices.length > 0) {
                ArrayList<LinkData> linkList = allLinks;
                if (filtered) linkList = filteredLinks;
                //calculateUnivKhov(chosenLinks(linkList, choices));
                int counter = 0;
                for (LinkData ld : linkList) counter = foxMilnorCheck(ld, counter);
                System.out.println("Total "+counter);
                //theButtons.calculateCompleteSInv(chosenLinks(linkList, choices));

                //theButtons.calculateSlThreeHom(chosenLinks(linkList, choices));

                //theButtons.calculateSlThreePol(chosenLinks(linkList, choices));
                //theButtons.calculateUnifiedKhov(chosenLinks(linkList, choices));
                /*
                for (LinkData ld : linkList) {
                    System.out.println(ld.redBLT);
                    System.out.println(ld.name+" "+ld.bltEPages(2, true));
                    if (ld.integralHomologyExists(true)) {
                        System.out.println("Char 0 mod 4 : "+ld.getTotalKhov(0, true)%4);
                        System.out.println("Char 2 mod 4 : "+ld.getTotalKhov(2, true)%4);
                        System.out.println("0th Betti : "+ld.getBettiKhov(0, 0, true));
                        int[] tot = ld.getBettiKhovSgn(0, true);
                        System.out.println("Even : "+tot[0]+"  Odd : "+tot[1]);
                        System.out.println();
                    }
                    Reidemeister.combineCrossings(ld.chosenLink());
                    ArrayList<Integer> tors = ld.torsionInGraded();
                    boolean printOut = false;
                    if (tors.size() > 1) printOut = true;
                    if (containsBigger(tors, 2)) printOut = true;
                    if (weirdSqOne(ld)) printOut = true;
                    checkBLT(ld);
                    checkSignature(ld);
                    checkJones(ld);
                    checkStable(ld);
                    if (printOut) System.out.println(ld.name+" "+ld.torsionInGraded());
                } // */
                /*ArrayList<String> names = new ArrayList<String>();
                ArrayList<ArrayList<String>> data = new ArrayList<ArrayList<String>>();
                for (LinkData ld : linkList) {
                    HalfPolynomial poly = new HalfPolynomial(null, ld.jones);
                    String jp = poly.toReducedString();
                    String jpm = poly.mirror().toReducedString();
                    String rel = jp;
                    if (jpm.compareTo(jp) > 0) rel = jpm;
                    rel = rel.replaceAll(",", "_");
                    int fc = rel.indexOf("_");
                    int sc = rel.indexOf("_", fc+1);
                    int tc = rel.indexOf("_", sc+1);
                    String name = rel.substring(0, tc);
                    String pd = LinkSaver.pdInfo(ld.chosenLink().breakUp(),
                            LinkSaver.transfer(ld.chosenLink().breakUp()));
                    String lData = ld.name+", "+pd.replaceAll(",", ".")+", "+rel;
                    if (names.contains(name)) {
                        int p = names.indexOf(name);
                        ArrayList<String> guys = data.get(p);
                        guys.add(lData);
                    }
                    else {
                        names.add(name);
                        ArrayList<String> newGuy = new ArrayList<String>();
                        newGuy.add(lData);
                        data.add(newGuy);
                    }
                }//
                for (int i = 0; i < names.size(); i++) {
                    System.out.println(names.get(i)+" "+data.get(i).size()+"  "+i);
                    //System.out.println(data.get(i));
                }
                JFileChooser chooser = new JFileChooser();
                int val = chooser.showSaveDialog(this);
                if (val == JFileChooser.APPROVE_OPTION) {
                    try {
                        File folder = chooser.getCurrentDirectory();
                        for (int i = 0; i < names.size(); i++) {
                            String name = names.get(i);
                            int fc = name.indexOf("_");
                            int sc = name.indexOf("_", fc+1);
                            if (sc == -1) sc = name.length();
                            String dir = folder.toString()+"/"+name.substring(0, sc);
                            new File(dir).mkdirs();
                            File file = new File(dir+"/"+name+".csv");
                            if (file.isFile()) {
                                System.out.println(name+" exists");
                                try (FileWriter fw = new FileWriter(file, true)) {
                                    try (PrintWriter pw = new PrintWriter(fw)) {
                                        for (String dat : data.get(i)) pw.println(dat);
                                    }
                                }
                            }
                            else {
                                System.out.println(name+" doesn't exist");
                                try (FileWriter fw = new FileWriter(file)) {
                                    try (PrintWriter pw = new PrintWriter(fw)) {
                                        for (String dat : data.get(i)) pw.println(dat);
                                    }
                                }
                            }
                        }
                    }
                    catch (IOException e) {

                    }
                }// */
            }
        }
    }

    private int foxMilnorCheck(LinkData data, int counter) {
        if (data.alex != null && data.alex.contains("F")) counter++;
        //System.out.print(data.name+" "+poly);System.out.println(" "+poly.foxMilnor());
        return counter;
    }

    /*private void checkSignature(LinkData ld) {
        if (ld.signature == null) return;
        int sgn = Integer.parseInt(ld.signature);
        if (sgn != ld.sInvariant(3)) System.out.println(ld.name);
    }

    private void checkStable(LinkData ld) {
        int[] sq = ld.getSqTwo(2);
        if (sq == null) return;
        int s = ld.sInvariant(2);
        if (sq[1] != s || sq[3] != s) {
            if (sq[0] != s || sq[2] != s)
                System.out.println(ld.name+"  "+sq[0]+" "+sq[1]+" "+sq[2]+" "+sq[3]+"  "+s);
        }
    }

    private void checkJones(LinkData ld) {
        if ("2,1,0,1,-1".equals(ld.jones) || "-8,-1,1,0,1".equals(ld.jones))
            System.out.println(ld.name+" as Trefoil");
    }

    private void checkBLT(LinkData ld) {
        int[] info = ld.bltInfo(0, false);
        if (info == null) return;
        int collapse = info.length/2;
        if (collapse >= 3) System.out.println(ld.name+" collapses at E_"+collapse);
    }

    private boolean weirdSqOne(LinkData ld) {
        int[] sqeven = ld.getSqOne(true);
        if (sqeven == null) return false;
        boolean weird = (sqeven[0] != sqeven[1] | sqeven[2] != sqeven[3]);
        if (weird) System.out.println(ld.name+" "+ld.sqEven);
        return false;
    }

    private boolean containsBigger(ArrayList<Integer> tors, int ref) {
        boolean cont = false;
        for (Integer t : tors) if (t > ref) cont = true;
        return cont;
    }// */

    private ArrayList<LinkData> getNewLink() {
        JDialog fram = new JDialog(new JFrame(),"New Link", true);
        fram.setSize(400,300);
        fram.setLocationRelativeTo(this);
        fram.setResizable(false);
        fram.setLayout(new GridLayout(7,2));
        JButton torLink = new JButton("Torus Link");
        JButton preLink = new JButton("Pretzel Link");
        JButton dtKnot  = new JButton("DT Code");
        JButton pdLink  = new JButton("PD Code");
        JButton gaLink  = new JButton("Gauss Code");
        JButton adtLink = new JButton("alphabetical DT Code");
        JButton braLink = new JButton("Braid Code");
        JButton conLink = new JButton("Concatenate Links");
        JButton splLink = new JButton("Split Union");
        JButton mirLink = new JButton("Mirror Links");
        JButton unLink  = new JButton("Unlink");
        JButton orLink  = new JButton("Change Orientation");
        JButton whDoub  = new JButton("Whitehead Doubles");
        JButton cancelB = new JButton("Cancel");
        torLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                torLink.setEnabled(false);
            }
        });
        preLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                preLink.setEnabled(false);
            }
        });
        dtKnot.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                dtKnot.setEnabled(false);
            }
        });
        pdLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                pdLink.setEnabled(false);
            }
        });
        adtLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                adtLink.setEnabled(false);
            }
        });
        braLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                braLink.setEnabled(false);
            }
        });
        conLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                conLink.setEnabled(false);
            }
        });
        splLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                splLink.setEnabled(false);
            }
        });
        mirLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                mirLink.setEnabled(false);
            }
        });
        cancelB.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
            }
        });
        gaLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                gaLink.setEnabled(false);
            }
        });
        unLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                unLink.setEnabled(false);
            }
        });
        orLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                orLink.setEnabled(false);
            }
        });
        whDoub.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                whDoub.setEnabled(false);
            }
        });
        JPanel torPanel = new JPanel();
        JPanel prePanel = new JPanel();
        JPanel dtcPanel = new JPanel();
        JPanel pdcPanel = new JPanel();
        JPanel gauPanel = new JPanel();
        JPanel adtPanel = new JPanel();
        JPanel braPanel = new JPanel();
        JPanel conPanel = new JPanel();
        JPanel splPanel = new JPanel();
        JPanel mirPanel = new JPanel();
        JPanel unPanel  = new JPanel();
        JPanel orPanel  = new JPanel();
        JPanel whPanel  = new JPanel();
        JPanel canPanel = new JPanel();
        torPanel.add(torLink);
        prePanel.add(preLink);
        dtcPanel.add(dtKnot);
        pdcPanel.add(pdLink);
        gauPanel.add(gaLink);
        adtPanel.add(adtLink);
        braPanel.add(braLink);
        conPanel.add(conLink);
        splPanel.add(splLink);
        mirPanel.add(mirLink);
        unPanel.add(unLink);
        orPanel.add(orLink);
        whPanel.add(whDoub);
        canPanel.add(cancelB);
        fram.add(torPanel);
        fram.add(prePanel);
        fram.add(braPanel);
        fram.add(dtcPanel);
        fram.add(pdcPanel);
        fram.add(gauPanel);
        fram.add(adtPanel);
        fram.add(conPanel);
        fram.add(splPanel);
        fram.add(mirPanel);
        fram.add(unPanel);
        fram.add(orPanel);
        fram.add(whPanel);
        fram.add(canPanel);
        fram.setVisible(true);
        ArrayList<LinkData> links = new ArrayList<LinkData>();
        if (!torLink.isEnabled()) links.add(enterTorusLink());
        if (!preLink.isEnabled()) links.add(enterPretzelLink());
        if (!dtKnot.isEnabled())  links.add(LinkCreator.enterDTCode(null, null, true, this, comparer));
        if (!pdLink.isEnabled())  links.add(LinkCreator.enterPDCode(null, null, true, this, comparer));
        if (!gaLink.isEnabled())  links.add(LinkCreator.enterGaussCode(this, comparer));
        if (!adtLink.isEnabled()) links.add(LinkCreator.enterADTCode(null, null, true, this, comparer));
        if (!braLink.isEnabled()) links.add(LinkCreator.enterBraidCode(null, null, this, true, comparer));
        if (!conLink.isEnabled()) links.add(concatenateLinks());
        if (!splLink.isEnabled()) links.add(disjointUnion());
        if (!mirLink.isEnabled()) links = mirrorLink();
        if (!unLink.isEnabled())  links.add(unLink());
        if (!orLink.isEnabled())  links.add(orLink());
        if (!whDoub.isEnabled())  links = whiteLink();
        return links;
    }

    private LinkData concatenateLinks() {
        ArrayList<LinkData> theLinks = allLinks;
        DefaultListModel<String> listModel = listModelAll;
        if (filtered) {
            theLinks = filteredLinks;
            listModel = listModelFiltered;
        }
        if (theLinks.isEmpty()) return null;
        UnionKnot uKnot = new UnionKnot(this, "Concatenate Links", theLinks, listModel, true);
        if (uKnot.knot == null) return null;
        String name = uKnot.name;
        return new LinkData(name,uKnot.knot.girthMinimize(),comparer);
    }

    private LinkData disjointUnion() {
        ArrayList<LinkData> theLinks = allLinks;
        DefaultListModel<String> listModel = listModelAll;
        if (filtered) {
            theLinks = filteredLinks;
            listModel = listModelFiltered;
        }
        if (theLinks.isEmpty()) return null;
        UnionKnot uKnot = new UnionKnot(this, "Split Union", theLinks, listModel, false);
        if (uKnot.knot == null) return null;
        String name = uKnot.name;
        return new LinkData(name,uKnot.knot.girthMinimize(),comparer);
    }

    private LinkData enterTorusLink() {
        JDialog fram = new JDialog(new JFrame(), "Torus Link", true);
        fram.setSize(400, 150);
        fram.setLocationRelativeTo(this);
        fram.setLayout(new BorderLayout());
        fram.setResizable(false);
        SpinnerNumberModel modelq = new SpinnerNumberModel(5,2,60,1);
        JSpinner spinnerq = new JSpinner(modelq);
        JLabel labep = new JLabel("Enter a p-value :");
        SpinnerNumberModel modelp = new SpinnerNumberModel(3,2,10,1);
        JSpinner spinnerp = new JSpinner(modelp);
        JLabel labeq = new JLabel("Enter a q-value :");
        JPanel panep = new JPanel();
        panep.add(labep);
        panep.add(spinnerp);
        JPanel paneq = new JPanel();
        paneq.add(labeq);
        paneq.add(spinnerq);
        JButton ok = new JButton("OK");
        JButton cancel = new JButton("Cancel");
        JPanel paneb = new JPanel();
        paneb.add(ok);
        paneb.add(cancel);
        fram.add(panep,BorderLayout.NORTH);
        fram.add(paneq,BorderLayout.CENTER);
        fram.add(paneb,BorderLayout.SOUTH);
        ok.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                ok.setEnabled(false);
                fram.dispose();
            }
        });
        cancel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
            }
        });
        fram.setVisible(true);
        if (ok.isEnabled()) return null;
        int p = modelp.getNumber().intValue();
        int q = modelq.getNumber().intValue();
        Link tLink = torusKnot(q,p);
        LinkData tKnot = new LinkData("T("+p+","+q+")", tLink,comparer);
        return tKnot;
    }

    private Link torusKnot(int p, int q) {
        int prod = p * (q-1);
        int [] crossings = new int[prod];
        for (int k = 0 ; k < prod; k++) {
            crossings[k] = 1;
        }
        int [][] paths = new int[prod][4];
        for (int i = 0; i < p; i++) {
            int u = i * (q-1);
            paths[u][0] = (2*i*(q-1)) + 2;
            paths[u][1] = ((2*i+1)*(q-1)) + 2;
            paths[u][2] = (2*(i+1)*(q-1)) + 1;
            paths[u][3] = (2*i*(q-1)) + 1;
        }
        paths[(p-1)*(q-1)][2] = 1;
        for (int j = 1; j < q-2; j++) {
            for (int i = 0; i < p ; i++) {
                int u = i * (q-1) + j;
                paths[u][0] = (2*i*(q-1)) + j + 2;
                paths[u][1] = ((2*i)+1)*(q-1) + j + 2;
                paths[u][2] = ((2*i)+2)*(q-1) + j + 1;
                paths[u][3] = ((2*i)+1)*(q-1) + j + 1;
            }
            paths[(p-1)*(q-1)+j][2] = j + 1;
        }
        for (int i = 0; i < p; i++) {
            int u = i * (q-1) + (q - 2);
            paths[u][0] = (2*i*(q-1)) + q;
            paths[u][1] = (2*(i+1)*(q-1)) + q;
            paths[u][2] = ((2*i+2)*(q-1)) + q - 1;
            if (q > 2) paths[u][3] = ((2*i+1)*(q-1)) + q - 1;
            else paths[u][3] = ((2*i+1)*(q-1));
        }
        paths[p*(q-1)-1][1] = q;
        paths[p*(q-1)-1][2] = q-1;
        Link tLink = new Link(crossings,paths);
        return tLink;
    }

    private ArrayList<LinkData> whiteLink() {
        if (allLinks.isEmpty()) return new ArrayList<LinkData>(0);
        MirrorLink frame = new MirrorLink(this, "Whitehead Doubles", listModelAll);
        frame.setUpStuff();
        if (frame.getChosen() == -1) return new ArrayList<LinkData>(0);
        ArrayList<LinkData> theLinks = new ArrayList<LinkData>();
        for (int i : frame.getAllChosen()) {
            int t = frame.getTwists();
            Link[] theDoubles = allLinks.get(i).chosenLink().whiteheadDoubles(2*t);
            if (theDoubles != null) {
                t = t +  allLinks.get(i).chosenLink().writhe();
                String name = "W("+allLinks.get(i).name+", "+t+", ";
                theLinks.add(mirrorData(name+"+)", theDoubles[0], allLinks.get(i)));
                theLinks.add(mirrorData(name+"-)", theDoubles[1], allLinks.get(i)));
            }
        }
        return theLinks;
    }

    private ArrayList<LinkData> mirrorLink() {
        if (allLinks.isEmpty()) return new ArrayList<LinkData>(0);
        MirrorLink frame = new MirrorLink(this, "Mirror Link", true, false, listModelAll);
        frame.setUpStuff();
        if (frame.getChosen() == -1) return new ArrayList<LinkData>(0);
        ArrayList<LinkData> theLinks = new ArrayList<LinkData>();
        for (int i : frame.getAllChosen()) {
            Link theMirror = allLinks.get(i).chosenLink().mirror();
            theLinks.add(mirrorData("-"+allLinks.get(i).name, theMirror, allLinks.get(i)));
        }
        return theLinks;
    }

    private LinkData mirrorData(String name, Link theMirror, LinkData origLink) {
        LinkData mirrorData = new LinkData(name, theMirror, comparer);
        if (origLink.sqEven != null) {
            int[] sqeven = origLink.getSqOne(true); // at the moment we don't mirror the odd sq1.
            int[] mqeven = new int[4];
            mqeven[0] = -sqeven[2];
            mqeven[1] = -sqeven[3];
            mqeven[2] = -sqeven[0];
            mqeven[3] = -sqeven[1];
            mirrorData.setSqOne(mqeven);
        }
        if (origLink.sinvariant != null) {
            int[][] sinv = origLink.sInvariants();
            for (int i = 0; i < sinv.length; i++) {
                mirrorData.setSInvariant(sinv[i][0], -sinv[i][1]);
            }
        }
        if (origLink.khovInfo == null) return mirrorData;
        mirrorData.khovInfo = new ArrayList<String>();
        long[][] data = origLink.getStartInfo(origLink.khovInfo, options.getPrimes());
        for (int i = 0; i < origLink.khovInfo.size(); i++) {
            String info = origLink.khovInfo.get(i);
            boolean reduced = false;
            if (info.charAt(0) == 'r') {
                reduced = true;
                if (mirrorData.redKhovHom == null) mirrorData.redKhovHom = new ArrayList<String>();
            }
            else if (mirrorData.unredKhovHom == null) mirrorData.unredKhovHom = new ArrayList<String>();
            ArrayList<String> homStrings = origLink.unredKhovHom;
            if (reduced) homStrings = origLink.redKhovHom;
            HomologyInfo homInfo = origLink.theHomology(data[i], homStrings).mirror();
            ArrayList<String> mirrorKhovs = mirrorData.unredKhovHom;
            if (reduced) mirrorKhovs = mirrorData.redKhovHom;
            for (QuantumCohomology qCoh : homInfo.getHomologies()) {
                mirrorKhovs.add(qCoh.toString());
            }
            mirrorData.khovInfo.add(info);
        }
        return mirrorData;
    }

    private LinkData orLink() {
        if (allLinks.isEmpty()) return null;
        OrientLink frame = new OrientLink(this, "Change Orientation Link", true, listModelAll, allLinks);
        frame.setUpStuff();
        if (frame.chosen == -1) return null;
        Link theLink = allLinks.get(frame.chosen).chosenLink().componentChoice(frame.comps,frame.orient);
        return new LinkData("V"+allLinks.get(frame.chosen).name, theLink,comparer);
    }

    private LinkData unLink() {
        JDialog fram = new JDialog(new JFrame(), "Unlink", true);
        fram.setSize(300, 130);
        fram.setLocationRelativeTo(this);
        fram.setLayout(new BorderLayout());
        fram.setResizable(false);
        JPanel choicePanel = new JPanel();
        JPanel buttonPanel = new JPanel();
        SpinnerNumberModel model = new SpinnerNumberModel(1,1,10,1);
        JSpinner spinner = new JSpinner(model);
        JLabel label = new JLabel("Number of components :");
        choicePanel.add(label);
        choicePanel.add(spinner);
        JButton ok = new JButton("OK");
        JButton cancel = new JButton("Cancel");
        buttonPanel.add(ok);
        buttonPanel.add(cancel);
        fram.add(choicePanel, BorderLayout.CENTER);
        fram.add(buttonPanel, BorderLayout.SOUTH);
        fram.add(new JPanel(), BorderLayout.NORTH);
        ok.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ok.setEnabled(false);
                fram.dispose();
            }
        });
        cancel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fram.dispose();
            }
        });
        fram.setVisible(true);
        if (ok.isEnabled()) return null;
        int comp = model.getNumber().intValue();
        Link newLink = new Link(comp);
        String name = "U_"+comp;
        return new LinkData(name,newLink,comparer);
    }

    private void setKnotInfoEmpty() {
        panelKnotInfo.removeAll();
        JLabel label = new JLabel(" ");
        label.setPreferredSize(new Dimension(790,600));
        panelKnotInfo.add(label);
        panelKnotInfo.revalidate();
        choice = -1;
    }

    private void setLinkInfo(LinkData theLink) {
        JLabel label = new JLabel(theLink.name, JLabel.CENTER);
        label.setFont(new Font("Sans Serif",Font.BOLD, 16));
        panelKnotInfo.removeAll();
        JPanel topInfo = new JPanel();
        String commStr = " ";
        if (theLink.comment != null) commStr = "Comment : "+theLink.comment;
        JLabel comment = new JLabel(commStr, JLabel.CENTER);
        topInfo.setLayout(new GridLayout(2,1));
        topInfo.add(label);
        topInfo.add(comment);
        topInfo.setPreferredSize(new Dimension(500,50));
        JPanel crossingInfo = new JPanel();
        setUpCrossingInfo(crossingInfo,theLink);
        JPanel diagInfo = new JPanel();
        diagInfo.setLayout(new GridLayout(1,5));
        JLabel diagLabel = new JLabel("Diagrams", JLabel.CENTER);
        JPanel diagChoice = new JPanel();
        int n = theLink.links.size();
        diagChoice.setLayout(new GridLayout(n,1));
        ButtonGroup diagramButtons = new ButtonGroup();
        ActionListener listener;
        listener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                JRadioButton btn = (JRadioButton) ae.getSource();
                int i = 0;
                try {
                    i = Integer.parseInt(btn.getName());
                }
                catch(NumberFormatException e) {

                }
                theLink.setChosen(i);
                setUpCrossingInfo(crossingInfo,theLink);
            }
        };
        for (int i = 0; i < n; i++) {
            int j = i+1;
            JRadioButton rButton = new JRadioButton("Diagram "+j);
            if (i == theLink.chosen()) rButton.setSelected(true);
            diagramButtons.add(rButton);
            diagChoice.add(rButton);
            rButton.setName(String.valueOf(i));
            rButton.addActionListener(listener);
        }
        JButton diagShow = new JButton("Show Diagram");
        JPanel diagShowP = new JPanel();
        diagShowP.add(diagShow, BorderLayout.CENTER);
        JButton diagDTCode = new JButton("DT-Code");
        JPanel diagDTCodeP = new JPanel();
        if (theLink.links.get(0).components() > 1) diagDTCode.setEnabled(false);
        diagShow.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showTheDiagram(theLink);
            }
        });
        diagDTCode.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String code = theLink.chosenLink().dowkerThistle();
                showString("DT Code", code, theLink.name);
            }
        });
        diagDTCodeP.add(diagDTCode,BorderLayout.SOUTH);
        JButton diagGauss = new JButton("Gauss-Code");
        JPanel diagGaussP = new JPanel();
        diagGauss.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String code = theLink.chosenLink().gaussCode();
                showString("Gauss Code", code, theLink.name);
            }
        });
        diagGaussP.add(diagGauss, BorderLayout.CENTER);
        JScrollPane diagPane = new JScrollPane(diagChoice);
        diagPane.setPreferredSize(new Dimension(80,80));
        diagInfo.add(diagLabel);
        diagInfo.add(diagPane);
        diagInfo.add(diagShowP);
        diagInfo.add(diagDTCodeP);
        diagInfo.add(diagGaussP);
        JPanel invPanel = new JPanel();
        JPanel signaturePanel = new JPanel();
        fillSignature(signaturePanel, theLink);
        invPanel.add(signaturePanel);
        JPanel rasmusPanel = new JPanel();
        fillRasmussen(rasmusPanel, theLink);
        invPanel.add(rasmusPanel);
        JPanel lipsarPanel = new JPanel();
        fillLipSar(lipsarPanel, theLink);
        invPanel.add(lipsarPanel);
        JPanel lipsarMorePanel = new JPanel();
        fillLipSarMore(lipsarMorePanel, theLink);
        invPanel.add(lipsarMorePanel);
        //JPanel lipsarCompPanel = new JPanel();
        //fillLipSarComp(lipsarCompPanel, theLink);
        //invPanel.add(lipsarCompPanel);
        JPanel lipsartwoPanel = new JPanel();
        fillLipSarTwo(lipsartwoPanel, theLink);
        invPanel.add(lipsartwoPanel);
        JPanel khovanovPanel = new JPanel();
        fillKhovanov(khovanovPanel, theLink);
        invPanel.add(khovanovPanel);
        JPanel bLTPanel = new JPanel();
        fillBLT(bLTPanel, theLink);
        invPanel.add(bLTPanel);
        JPanel stablePanel = new JPanel();
        fillStablePanel(stablePanel, theLink);
        invPanel.add(stablePanel);
        JPanel polyPanel = new JPanel();
        fillPolyPanel(polyPanel, theLink);
        invPanel.add(polyPanel);
        invPanel.setPreferredSize(new Dimension(700,350));
        panelKnotInfo.add(topInfo);
        panelKnotInfo.add(diagInfo);
        panelKnotInfo.add(crossingInfo);
        panelKnotInfo.add(invPanel);
        panelKnotInfo.revalidate();
    }

    private void fillSignature(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1, 3));
        String sign = "  Signature : ";
        if (theLink.signature != null) sign = sign + theLink.signature;
        JPanel signPanel = new JPanel();
        signPanel.add(new JLabel(sign));
        panel.add(signPanel);
        String det = "  Determinant : ";
        if (theLink.determinant != null) det = det + theLink.determinant;
        JPanel detPanel = new JPanel();
        detPanel.add(new JLabel(det));
        panel.add(detPanel);
        String grs = "  Graded s-Invariant : ";
        if (theLink.grsinv != null) grs = grs + theLink.grsinv;
        JPanel grPanel = new JPanel();
        grPanel.add(new JLabel(grs));
        panel.add(grPanel);
        panel.setPreferredSize(panelDimSm);
    }

    private void fillRasmussen(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1, 3));
        int[][] data = theLink.sInvariants();
        Integer ras = getValue(data, 0);
        String rasmus = "     Rasmussen invariant : ";
        if (ras != null) rasmus = rasmus+ras;
        JLabel rasmusLabel = new JLabel(rasmus);
        ras = getValue(data,2);
        String sinv = "     s-Invariant mod 2 : ";
        if (ras != null) sinv = sinv+ras;
        JLabel sinvmodtwoLabel = new JLabel(sinv);
        JButton otherInvButton = new JButton("Other s-Invariants");
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(otherInvButton);
        otherInvButton.setEnabled(theLink.otherSInvariants());
        otherInvButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                displayOtherSInvariants(theLink);
            }
        });
        panel.add(rasmusLabel);
        panel.add(sinvmodtwoLabel);
        panel.add(buttonPanel);
        panel.setPreferredSize(panelDimLg);
    }

    private void fillLipSar(JPanel panel, LinkData link) {
        panel.setLayout(new GridLayout(1,3));
        JLabel LpSkLabel = new JLabel("LS-Invariants ");
        JPanel LpSkPanel = new JPanel();
        LpSkPanel.add(LpSkLabel);
        panel.add(LpSkPanel);
        String sq1Inv = "Sq^1-even : ";
        if (link.sqEven != null) sq1Inv = sq1Inv+link.sqEven;
        JLabel sq1Label = new JLabel(sq1Inv);
        JPanel sq1Panel = new JPanel();
        sq1Panel.add(sq1Label);
        panel.add(sq1Panel);
        String sq1oInv = "Sq^1-odd : ";
        if (link.sqOdd != null) sq1oInv = sq1oInv+link.sqOdd;
        JLabel sq1oLabel = new JLabel(sq1oInv);
        JPanel sq1oPanel = new JPanel();
        sq1oPanel.add(sq1oLabel);
        panel.add(sq1oPanel);
        panel.setPreferredSize(panelDimSm);
    }

    private void fillLipSarMore(JPanel panel, LinkData link) {
        panel.setLayout(new GridLayout(1,3));
        String unInv = "Complete LS-Inv : ";
        if (link.cmpinv != null) unInv = unInv+link.cmpinv;
        JLabel LpSkLabel = new JLabel(unInv);
        JPanel LpSkPanel = new JPanel();
        LpSkPanel.add(LpSkLabel);
        panel.add(LpSkPanel);
        String sq1Inv = "Sq^1-sum : ";
        if (link.beta != null) sq1Inv = sq1Inv+link.beta;
        JLabel sq1Label = new JLabel(sq1Inv);
        JPanel sq1Panel = new JPanel();
        sq1Panel.add(sq1Label);
        panel.add(sq1Panel);
        String bsInv = "BLS-odd : ";
        if (link.bsOdd != null) bsInv = bsInv+link.bsOdd;
        JLabel bsLabel = new JLabel(bsInv);
        JPanel bsPanel = new JPanel();
        bsPanel.add(bsLabel);
        panel.add(bsPanel);
        panel.setPreferredSize(panelDimSm);
    }

    /*private void fillLipSarComp(JPanel panel, LinkData link) {
        panel.setLayout(new GridLayout(1,3));
        String unInv = "Complete LS-Invariants";
        JLabel LpSkLabel = new JLabel(unInv);
        JPanel LpSkPanel = new JPanel();
        LpSkPanel.add(LpSkLabel);
        panel.add(LpSkPanel);
        String redInv = "Reduced : ";
        if (link.cmpinv != null) redInv = redInv+link.cmpinv;
        JLabel sq1Label = new JLabel(redInv);
        JPanel sq1Panel = new JPanel();
        sq1Panel.add(sq1Label);
        panel.add(sq1Panel);
        String bsInv = "Unreduced : ";
        if (link.cuninv != null) bsInv = bsInv+link.cuninv;
        JLabel bsLabel = new JLabel(bsInv);
        JPanel bsPanel = new JPanel();
        bsPanel.add(bsLabel);
        panel.add(bsPanel);
        panel.setPreferredSize(panelDimSm);
    }// */

    private void fillLipSarTwo(JPanel panel, LinkData Link) {
        panel.setLayout(new GridLayout(1,3));
        String sq2Inv = "Sq^2-even : ";
        if (Link.sqtEven != null) sq2Inv = sq2Inv+Link.sqtEven;
        JLabel sq1Label = new JLabel(sq2Inv);
        JPanel sq1Panel = new JPanel();
        sq1Panel.add(sq1Label);
        panel.add(sq1Panel);
        String sqodInv = "Sq^2_0-odd : ";
        if (Link.sqtOdd != null) sqodInv = sqodInv+Link.sqtOdd;
        JLabel sqodLabel = new JLabel(sqodInv);
        JPanel sqodPanel = new JPanel();
        sqodPanel.add(sqodLabel);
        panel.add(sqodPanel);
        String sqoeInv = "Sq^2_1-odd : ";
        if (Link.sqtOde != null) sqoeInv = sqoeInv+Link.sqtOde;
        JLabel sqoeLabel = new JLabel(sqoeInv);
        JPanel sqoePanel = new JPanel();
        sqoePanel.add(sqoeLabel);
        panel.add(sqoePanel);
        panel.setPreferredSize(panelDimSm);
    }

    private void displayOtherSInvariants(LinkData theLink) {
        ViewSInvariants viewer = new ViewSInvariants(theLink,options);
        viewer.setupFrame();
        viewer.setLocationRelativeTo(this);
        viewer.setVisible(true);
    }

    private Integer getValue(int[][] data, int field) {
        boolean found = false;
        int i = 0;
        while (!found && i < data.length) {
            if (data[i][0] == field) found = true;
            else i++;
        }
        if (found) return data[i][1];
        return null;
    }

    private void fillPolyPanel(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1,2));
        String polyString = "Polynomials";
        JLabel polyLabel = new JLabel(polyString);
        JPanel polyPanel = new JPanel();
        polyPanel.add(polyLabel);
        panel.add(polyPanel);
        JButton jonesButton = new JButton("Jones");
        jonesButton.setEnabled(theLink.jones != null);
        JButton alexButton = new JButton("Alexander");
        alexButton.setEnabled(theLink.alex != null);
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(jonesButton);
        buttonPanel.add(alexButton);
        panel.add(buttonPanel);
        panel.setPreferredSize(panelDimLg);
        jonesButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                showPolynomial("Jones Polynomial of "+theLink.name, "Jones", theLink.jones, "q");
            }
        });
        alexButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                showPolynomial("Alexander Polynomial of "+theLink.name, "Alexander",
                        theLink.alex, "t");
            }
        });
    }

    private void fillStablePanel(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1, 2));
        String name = "Stable Homotopy Type";
        JLabel nameLabel = new JLabel(name);
        JPanel namePanel = new JPanel();
        namePanel.add(nameLabel);
        JButton evenButton = new JButton("even");
        evenButton.setEnabled(theLink.showStEvenButton());
        char delta = (char) 949; // it's really epsilon
        JButton oddButton = new JButton("odd ("+delta+"=0)");
        oddButton.setEnabled(theLink.showStOddButton(0));
        JButton odeButton = new JButton("odd ("+delta+"=1)");
        odeButton.setEnabled(theLink.showStOddButton(1));
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(evenButton);
        buttonPanel.add(oddButton);
        buttonPanel.add(odeButton);
        panel.add(namePanel);
        panel.add(buttonPanel);
        panel.setPreferredSize(panelDimLg);
        evenButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                showStableType(theLink, 2);
            }
        });
        oddButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                showStableType(theLink, 0);
            }
        });
        odeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                showStableType(theLink, 1);
            }
        });
    }

    private void fillBLT(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1,2));
        String khovHom = "Bar Natan-Lee-Turner Spectral Sequence";
        JLabel akhovHomLabel = new JLabel(khovHom);
        JPanel akhovHomPanel = new JPanel();
        akhovHomPanel.add(akhovHomLabel);
        panel.add(akhovHomPanel);
        JButton aunredButton = new JButton("unreduced");
        aunredButton.setEnabled(theLink.showBLTButton(false));
        JPanel aunredPanel = new JPanel();
        aunredPanel.add(aunredButton);
        JButton aredButton = new JButton("reduced");
        aredButton.setEnabled(theLink.showBLTButton(true));
        aunredPanel.add(aredButton);
        panel.add(aunredPanel);
        panel.setPreferredSize(panelDimLg);
        aunredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showBLT(theLink, false);
            }
        });
        aredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showBLT(theLink, true);
            }
        });
    }

    private void fillOddKhov(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1,2));
        String khovHom = "Odd Khovanov Homology";
        JLabel akhovHomLabel = new JLabel(khovHom);
        JPanel akhovHomPanel = new JPanel();
        akhovHomPanel.add(akhovHomLabel);
        panel.add(akhovHomPanel);
        JButton aunredButton = new JButton("unreduced");
        aunredButton.setEnabled(theLink.showOddKhovButton());
        JPanel aunredPanel = new JPanel();
        aunredPanel.add(aunredButton);
        JButton aredButton = new JButton("reduced");
        aredButton.setEnabled(theLink.showOddKhovButton());
        aunredPanel.add(aredButton);
        panel.add(aunredPanel);
        panel.setPreferredSize(panelDimLg);
        aunredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showOddHomology(theLink,false);
            }
        });
        aredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showOddHomology(theLink,true);
            }
        });// */
    }

    private void fillKhovanov(JPanel panel, LinkData theLink) {
        panel.setLayout(new GridLayout(1,3));
        String khovHom = "Khovanov Cohomology";
        JLabel akhovHomLabel = new JLabel(khovHom);
        JPanel akhovHomPanel = new JPanel();
        akhovHomPanel.add(akhovHomLabel);
        panel.add(akhovHomPanel);
        JButton aunredButton = new JButton("even");
        aunredButton.setEnabled(theLink.showKhovHomButton(false));
        JPanel aunredPanel = new JPanel();
        aunredPanel.add(aunredButton);
        JButton aredButton = new JButton("even reduced");
        aredButton.setEnabled(theLink.showKhovHomButton(true));
        aunredPanel.add(aredButton);
        JButton ounredButton = new JButton("odd");
        ounredButton.setEnabled(theLink.showOddKhovButton());
        JPanel ounredPanel = new JPanel();
        ounredPanel.add(ounredButton);
        JButton oredButton = new JButton("odd reduced");
        oredButton.setEnabled(theLink.showOddKhovButton());
        ounredPanel.add(oredButton);
        panel.add(aunredPanel);
        panel.add(ounredPanel);
        panel.setPreferredSize(panelDimLg);
        aunredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showHomology(theLink,false);
            }
        });
        aredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showHomology(theLink,true);
            }
        });
        ounredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showOddHomology(theLink,false);
            }
        });
        oredButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showOddHomology(theLink,true);
            }
        });
    }

    private void showOddHomology(LinkData theLink, boolean reduced) {
        ViewOddHomology viewer = new ViewOddHomology(theLink, reduced, options);
        String title;
        if (reduced) title = "Reduced Odd Khovanov Homology of "+theLink.name;
        else title = "Unreduced Odd Khovanov Homology of "+theLink.name;
        viewer.setUpStuff(title);
    }

    private void showHomology(LinkData theLink, boolean reduced) {
        ViewCohomology viewer = new ViewCohomology(theLink, reduced, options);
        String title;
        if (reduced) title = "Reduced Khovanov Cohomology of "+theLink.name;
        else title = "Unreduced Khovanov Cohomology of "+theLink.name;
        viewer.setUpStuff(title);
    }

    private void showBLT(LinkData theLink, boolean reduced) {
        String title = "Bar Natan-Lee-Turner Spectral Sequence of "+theLink.name;
        if (reduced) title = "Reduced "+title;
        else title = "Unreduced "+title;
        ViewBLTInvariant viewer = new ViewBLTInvariant(title, this, theLink, reduced, options);
        viewer.setUp();
    }

    private void showStableType(LinkData theLink, int odd) {
        ViewStableType viewer = new ViewStableType(theLink, this, odd, options);
        String title = " Stable Homotopy Type of "+theLink.name;
        if (odd <= 1) title = "Odd ("+(char)949+"="+odd+")"+title;
        else title = "Even"+title;
        viewer.setupViewer(title);
    }

    private void showPolynomial(String title, String altTitle, String code, String label) {
        StringFrame fram = new StringFrame(title, true);
        HalfPolynomial poly = new HalfPolynomial(new String[] {label}, code);
        code = poly.toString();
        fram.setupStuff(this, altTitle, code);
        poly.setLatex(true);
        fram.setAlternative(poly.toString());
    }

    private void showString(String title, String code, String name) {
        StringFrame fram = new StringFrame(title+" of "+name, false);
        fram.setupStuff(this, title, code);
    }

    private void setUpCrossingInfo(JPanel panel, LinkData theLink) {
        panel.removeAll();
        panel.setLayout(new GridLayout(1,4));
        JPanel panel1 = new JPanel();
        JPanel panel2 = new JPanel();
        JPanel panel3 = new JPanel();
        JPanel panel4 = new JPanel();
        int bc = 1 + theLink.chosenLink().basecomponent();
        JLabel compLabel = new JLabel("Components : "+theLink.chosenLink().components()+" ("+bc+")");
        int[] crsigns = theLink.chosenLink().crossingSigns();
        int writhe = crsigns[0]-crsigns[1];
        JLabel writLabel = new JLabel("Writhe : "+writhe);
        int crossngs = crsigns[0]+crsigns[1];
        int crossngz = theLink.chosenLink().crossingLength();
        JLabel crosLabel = new JLabel("Crossings : "+crossngs+"/"+crossngz);
        int girth = theLink.chosenLink().maxGirth();
        int tgirth = theLink.chosenLink().totalGirth();
        JLabel girtLabel = new JLabel("Girth : "+girth+"/"+tgirth);
        panel1.add(compLabel);
        panel2.add(writLabel);
        panel3.add(crosLabel);
        panel4.add(girtLabel);
        panel.add(panel1);
        panel.add(panel2);
        panel.add(panel3);
        panel.add(panel4);
        panel.setPreferredSize(panelDimSm);
        panel.revalidate();
    }

    private void showTheDiagram(LinkData theLink) {
        if (options.getGridDiagram() == 0) {
            showGridDiagram(theLink);
            return;
        }
        int number = theLink.chosen()+1;
        DiagramFrame frame = new DiagramFrame(theLink.name+" - Diagram "+number, options);
        frame.setSize(524, 600);
        frame.setMinimumSize(new Dimension(524, 600));
        frame.setLocationRelativeTo(this);
        frame.addBasics();
        frame.setVisible(true);
        ShowDiagram showDiagram = new ShowDiagram(theLink, frame, options);
        showDiagram.start();
    }

    private void showGridDiagram(LinkData theLink) {
        int number = theLink.chosen()+1;
        GridDiagramFrame frame = new GridDiagramFrame(theLink.name+" - Diagram "+number,
                this, options, theLink);
        frame.setUp();
        ShowGridDiagram showDiagram = new ShowGridDiagram(theLink, frame);
        showDiagram.start();
    }

    private LinkData enterPretzelLink() {
        JDialog fram = new JDialog(new JFrame(), "Pretzel Link", true);
        fram.setSize(260,250);
        fram.setLocationRelativeTo(this);
        fram.setLayout(new BorderLayout());
        fram.setResizable(false);
        SpinnerNumberModel modelp = new SpinnerNumberModel(3,2,12,1);
        JSpinner spinnerp = new JSpinner(modelp);
        JLabel PreLabel = new JLabel("No. of Pretzels");
        JPanel PrePanel = new JPanel();
        PrePanel.add(PreLabel);
        PrePanel.add(spinnerp);
        fram.add(PrePanel, BorderLayout.WEST);
        SpinnerNumberModel[] models = new SpinnerNumberModel[12];
        JSpinner[] spinners = new JSpinner[12];
        JPanel SpinPanel = new JPanel();
        for (int i = 0; i < 12; i++) {
            models[i] = new SpinnerNumberModel(2,-999,999,1);
            spinners[i] = new JSpinner(models[i]);
            spinners[i].setPreferredSize(new Dimension(50,24));
        }
        addSpinners(SpinPanel, spinners,3);
        JScrollPane SpinPane = new JScrollPane(SpinPanel);
        spinnerp.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                int k = modelp.getNumber().intValue();
                addSpinners(SpinPanel,spinners,k);
            }
        });
        SpinPane.setPreferredSize(new Dimension(100,150));
        JPanel ButtonPanel = new JPanel();
        JButton OKButton = new JButton("OK");
        JButton CancelButton = new JButton("Cancel");
        OKButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
                spinnerp.setEnabled(false);
            }
        });
        CancelButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                fram.dispose();
            }
        });
        ButtonPanel.add(OKButton);
        ButtonPanel.add(CancelButton);
        fram.add(SpinPane, BorderLayout.EAST);
        fram.add(ButtonPanel, BorderLayout.SOUTH);
        fram.setVisible(true);
        LinkData preLink = null;
        if (!spinnerp.isEnabled()) {
            int k = modelp.getNumber().intValue();
            int[] crossings = new int[k];
            int[][] paths = new int[k][4];
            for (int u = 0; u < k; u++) crossings[u] = models[u].getNumber().intValue();
            String name = "P("+crossings[0];
            for (int u = 1; u < k; u++) name = name+","+crossings[u];
            name = name +")";
            for (int u = 0; u < k; u++) {
                paths[u][0] = 2*u + 1;
                paths[u][1] = 2*u + 2;
                paths[u][2] = 2*u;
                paths[u][3] = 2*u - 1;
            }
            paths[0][2] = 2*k;
            paths[0][3] = 2*k - 1;
            Link link = new Link(crossings,paths);
            preLink = new LinkData(name,link,comparer);
        }
        return preLink;
    }

    private void addSpinners(JPanel SpinPanel, JSpinner[] spinners, int k) {
        SpinPanel.removeAll();
        SpinPanel.setLayout(new GridLayout(k,1));
        for (int i = 0; i < k; i++) {
            JPanel panel = new JPanel();
            panel.add(spinners[i]);
            SpinPanel.add(panel);
        }
        SpinPanel.revalidate();
    }

    private void exportLinks() {
        ArrayList<LinkData> theLinks = allLinks;
        if (filtered) theLinks = filteredLinks;
        LinkSaver exporter = new LinkSaver(theLinks, choices, options, this);
        exporter.export();

    }

    private void saveLinks() {
        ArrayList<LinkData> theLinks = allLinks;
        if (filtered) theLinks = filteredLinks;
        LinkSaver saver = new LinkSaver(theLinks, choices, options, this);
        saver.save();
    }

    private ArrayList<LinkData> loadLinks(boolean select) {
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        ArrayList<LinkData> theLinks = new ArrayList<LinkData>();
        JFileChooser chooser = new JFileChooser();
        chooser.setMultiSelectionEnabled(true);
        if (options.getLoadLinksFrom() != null) chooser.setCurrentDirectory(options.getLoadLinksFrom());
        FileNameExtensionFilter filtek = new FileNameExtensionFilter("KnotJob Links (*.kjb)", "kjb");
        FileNameExtensionFilter filtet = new FileNameExtensionFilter("TKnotJob Links (*.tkj)", "tkj");
        FileNameExtensionFilter filter = new FileNameExtensionFilter("SKnotJob Links (*.kts)", "kts");
        FileNameExtensionFilter filtex = new FileNameExtensionFilter("XKnotJob Links (*.gld)", "gld");
        FileNameExtensionFilter filtes = new FileNameExtensionFilter("KnotScape Knots (*.dtc)", "dtc");
        FileNameExtensionFilter filtey = new FileNameExtensionFilter("Planar Diagrams (*.txt)", "txt");
        FileNameExtensionFilter filteb = new FileNameExtensionFilter("Braid Diagrams (*.brd)", "brd");
        FileNameExtensionFilter filtea = new FileNameExtensionFilter("Alphabetical DT-Code (*.adc)", "adc");
        chooser.setFileFilter(filtek);
        chooser.addChoosableFileFilter(filter);
        chooser.addChoosableFileFilter(filtet);
        if (!select) chooser.addChoosableFileFilter(filtex);
        chooser.addChoosableFileFilter(filtes);
        chooser.addChoosableFileFilter(filtey);
        chooser.addChoosableFileFilter(filteb);
        chooser.addChoosableFileFilter(filtea);
        int val = chooser.showOpenDialog(this);
        if (val == JFileChooser.APPROVE_OPTION) {
            LoadDialog fram = new LoadDialog(this, "Loading Links", true);
            LinkLoader loading = new LinkLoader(chooser, fram, this, select);
            loading.start();
            fram.setUpStuff();
            theLinks = loading.getLinks();
        }
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        return theLinks;
    }

    private ArrayList<LinkData> chosenLinks(ArrayList<LinkData> linkList, int[] choices) {
        ArrayList<LinkData> theList = new ArrayList<LinkData>(choices.length);
        for (int c : choices) theList.add(linkList.get(c));
        return theList;
    }

    private void minimizeGirth(ArrayList<LinkData> theLinks) {
        GirthDialogWrap fram = new GirthDialogWrap(this);
        GirthMinimizer giMi = new GirthMinimizer(theLinks, options, fram);
        giMi.start();
        fram.setup(giMi);
        giMi.setCancelled(true);
    }

    private void combineCrossings(ArrayList<LinkData> theLinks) {
        for (LinkData link : theLinks) {
            Link oldLink = link.chosenLink();
            Link newLink = Reidemeister.combineCrossings(oldLink);
            if (newLink.crossingLength() < oldLink.crossingLength()) {
                link.links.add(newLink.girthMinimize());
            }
        }
    }

    private void minimizeDiagramCrossings(ArrayList<LinkData> theLinks) {
        CombDialogWrap fram = new CombDialogWrap(this);
        CombMinimizer coMi = new CombMinimizer(theLinks, fram);
        coMi.start();
        fram.setup(coMi);
    }

    private void setOptions(int page) {
        OptionDialog opts = new OptionDialog(this, "Options", true, options, page);
        opts.setUpStuff();
    }

    public Options getOptions() {
        return options;
    }

    private void editLink() {
        ArrayList<LinkData> theLinks = allLinks;
        if (filtered) theLinks = filteredLinks;
        LinkData theData = theLinks.get(choice);
        EditDialog editor = new EditDialog(this,"Edit "+theData.name,true,theData);
        editor.setupDialog();
        if (editor.isOkay()) {
            theData.name = editor.getNewName();
            theData.comment = editor.getComment();
            setLinkInfo(theData);
            DefaultListModel<String> listModel = listModelAll;
            if (filtered) listModel = listModelFiltered;
            listModel.setElementAt(theData.name, choice);
        }
    }

    public void setActiveFilter(Filter filter) {
        activeFilter = filter;
        setFilter();
    }

    private void setFilter() {
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        filteredLinks.clear();
        listModelFiltered.removeAllElements();
        for (LinkData link : allLinks) {
            if (activeFilter.linkIsFiltered(link)) {
                filteredLinks.add(link);
                listModelFiltered.addElement(link.name);
            }
        }
        if (filtered) labelLinkNumber.setText("Links : "+filteredLinks.size());
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public Comparer getComparer() {
        return comparer;
    }

    private void sortLinks() {
        SortDialog sorter = new SortDialog(this, "Sort Links by", true, comparer.isReversed());
        int sort = sorter.getSelected();
        if (sort < 0) return;
        comparer.setType(sort);
        comparer.setReversed(sorter.isReversed());
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        Collections.sort(filteredLinks);
        listModelFiltered.removeAllElements();
        for (LinkData link : filteredLinks) listModelFiltered.addElement(link.name);
        Collections.sort(allLinks);
        listModelAll.removeAllElements();
        for (LinkData link : allLinks) listModelAll.addElement(link.name);
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        setKnotInfoEmpty();
    }

    public void updateLink() {
        if (filtered) {
            if (0 <= choice && choice < filteredLinks.size())
                setLinkInfo(filteredLinks.get(choice));
        }
        else {
            if (0 <= choice) setLinkInfo(allLinks.get(choice));
        }
    }

}
