/*

Copyright (C) 2022 Dirk Schuetz <dirk.schuetz@durham.ac.uk>

This file is part of KnotJob.

KnotJob is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

KnotJob is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org.licenses/>.

 */

package knotjob;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import javax.swing.JFileChooser;
import knotjob.links.Link;
import knotjob.links.LinkCreator;

/**
 *
 * @author Dirk
 */
public class KnotFileConverter {
    
    int counter = 0;
    int max = 50000;
    int filemax = 200;
    int totfile = 0;
    int fold = 0;
    
    KnotFileConverter() {
        
    }
    
    public static void main(String[] args) {
        KnotFileConverter conv = new KnotFileConverter();
        conv.getStarted();
        
    }

    private void getStarted() {
        JFileChooser chooser = new JFileChooser();
        chooser.setMultiSelectionEnabled(true);
        int val = chooser.showOpenDialog(null);
        if (val == JFileChooser.APPROVE_OPTION) {
            File[] files = chooser.getSelectedFiles();
            File directory = chooser.getCurrentDirectory();
            String fileName = files[0].getName();
            if (fileName.endsWith(".csv")) extractNames(files[0], directory, fileName);
            if (fileName.endsWith(".names")) convertNames(files, directory);
        }
    }

    private String getName(String line, String det) {
        int end = line.indexOf(det);
        return line.substring(0, end);
    }

    private String getDTCode(String line) {
        int end = line.indexOf(",");
        String help = line.substring(end+1);
        end = help.indexOf(",");
        help = help.substring(end+1);
        end = help.indexOf(",");
        if (end < 0) return help;
        return help.substring(0, end);
    }

    private void createFile(String name, File directory, ArrayList<String> lines) {
        totfile++;
        if (totfile >= filemax) {
            fold++;
            totfile = 0;
        }
        //System.out.println(name+" "+fold+" "+totfile);
        String first = getName(lines.get(0), " ");
        String last = "";
        if (lines.size()>1) last = "-"+getName(lines.get(lines.size()-1), " ");
        saveNames(first+last+".names", directory+"/"+name+"_"+fold, lines);
    }

    private void extractNames(File file, File directory, String fileName) {
        int dot = fileName.indexOf(".");
        fileName = fileName.substring(0, dot);
        try (FileReader fr = new FileReader(file)) {
            BufferedReader in = new BufferedReader(fr);
            boolean keepreading = true;
            String line;
            in.readLine();
            ArrayList<String> lines = new ArrayList<String>();
            while (keepreading) {
                line = in.readLine();
                if (line == null) keepreading = false;
                if (keepreading) {
                    counter++;
                    if (counter > max) {
                        counter = 1;
                        createFile(fileName, directory, lines);
                        lines = new ArrayList<String>();
                    }
                    String name = getName(line, ",");
                    String dtcd = getDTCode(line);
                    lines.add(name+" "+dtcd);
                }
            }
            if (!lines.isEmpty()) createFile(fileName, directory, lines);
            fr.close();
        }
        catch (IOException io) {

        }
    }

    private void saveNames(String name, String directory, ArrayList<String> lines) {
        File direc = new File(directory);
        if (!direc.exists()) direc.mkdirs();
        File file = new File(directory+"/"+name);
        try (FileWriter fw = new FileWriter(file)) {
            try (PrintWriter pw = new PrintWriter(fw)) {
                for (String data : lines) pw.println(data);
            }
        }
        catch (IOException e) {
            
        }// */
        System.out.println("Saved "+file);
    }

    private void convertNames(File[] files, File directory) {
        for (File file : files) {
            ArrayList<String> lines = new ArrayList<String>();
            try (FileReader fr = new FileReader(file)) {
                BufferedReader in = new BufferedReader(fr);
                boolean keepreading = true;
                String line;
                while (keepreading) {
                    line = in.readLine();
                    if (line == null) keepreading = false;
                    if (keepreading) {
                        String name = getName(line, " ");
                        String dtcd = getDTCode(line, " ");
                        Link link = LinkCreator.obtainLink(dtcd).girthMinimize();
                        lines.add(name+" ="+LinkSaver.exportData(link));
                    }
                }
                fr.close();
            }
            catch (IOException io) {

            }
            String direc = ""+directory;
            String fileName = file.getName();
            int u = fileName.indexOf(".");
            saveNames(fileName.substring(0, u)+".txt", (String) direc, lines);
        }
    }

    private String getDTCode(String line, String det) {
        int pos = line.indexOf(" ");
        String code = line.substring(pos+1);
        String dtcode = "";
        for (char c : code.toCharArray()) {
            int ascii = (int) c;
            int val;
            if (ascii > 96) val = 2*(ascii - 96);
            else val = (-2) *(ascii - 64);
            dtcode = dtcode+val+" ";
        }
        return dtcode;
    }
}
