/*

Copyright (C) 2019-20 Dirk Schuetz <dirk.schuetz@durham.ac.uk>

This file is part of KnotJob.

KnotJob is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

KnotJob is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org.licenses/>.

 */

package knotjob.filters;

import knotjob.links.LinkData;

/**
 *
 * @author Dirk
 */
public class BLTFilter implements Filter {
    
    private String name;
    private final boolean reduced;
    private final int characteristic;
    private final int lowerBound;
    private final boolean bddAbove;
    private final int upperBound;
    
    public BLTFilter(String nm, boolean red, int chr, int lb, int ub, boolean bdd) {
        name = nm;
        reduced = red;
        characteristic = chr;
        lowerBound = lb;
        upperBound = ub;
        bddAbove = bdd;
        
    }

    @Override
    public boolean linkIsFiltered(LinkData link) {
        int pages = link.bltEPages(characteristic, reduced).size();
        if (pages < lowerBound) return false;
        if (!bddAbove) return true;
        return (pages <= upperBound);
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public void setName(String nm) {
        name = nm;
    }
    
}
